import React from 'react';
import Button from '@material-ui/core/Button';
import TextField from '@material-ui/core/TextField';
import { makeStyles } from '@material-ui/core/styles';
import './Add.scss';
import {useState, useEffect} from "react";
import {useDispatch, useSelector} from "react-redux";
import {actioCreateAd,origUrl} from '../../App.js';
import AddCircleIcon from '@material-ui/icons/AddCircle';
import {useDropzone} from 'react-dropzone';
import {
  DndContext,
  KeyboardSensor,
  PointerSensor,
  useSensor,
  useSensors, useDroppable
} from "@dnd-kit/core";
import { sortableKeyboardCoordinates, rectSortingStrategy, SortableContext, useSortable, horizontalListSortingStrategy  } from "@dnd-kit/sortable";
import { CSS } from "@dnd-kit/utilities";
import {arrayMoveImmutable} from 'array-move';


const SortableItem = (props) => {
  const {
    attributes,
    listeners,
    setNodeRef,
    transform,
    transition
  } = useSortable({ id: props.id });

  const itemStyle = {
    transform: CSS.Transform.toString(transform),
    transition,
    cursor: "grab",
  };
    
    const Render = props.render

  return (
    <div style={itemStyle} ref={setNodeRef} {...attributes} {...listeners}>
      <Render {...{[props.itemProp]:props.item}}/>
    </div>
  );
};

const Droppable = ({ id, items, itemProp, keyField, render }) => {
  const { setNodeRef } = useDroppable({ id });

  return (
    <SortableContext id={id} items={items} strategy={rectSortingStrategy}>
        {items.map((item) => (
          <SortableItem render={render} key={item[keyField]} id={item} 
                        itemProp={itemProp} item={item}/>
        ))}
    </SortableContext>
  );
};

function Dnd({items:startItems,render, itemProp, keyField, onChange, horizontal}) {
  const [items, setItems] = useState(
      startItems
  );
  useEffect(() => setItems(startItems), [startItems])

  useEffect(() => {
      if (typeof onChange === 'function'){
          onChange(items)
      }
  },[items])

  const sensors = useSensors(
      useSensor(PointerSensor,  { activationConstraint: { distance: 5 } }),
      useSensor(KeyboardSensor, {
          coordinateGetter: sortableKeyboardCoordinates
      })
  );

  const handleDragEnd = ({ active, over }) => {
      const activeIndex = active.data.current.sortable.index;
      const overIndex = over.data.current?.sortable.index || 0;

      setItems((items) => {
          return arrayMoveImmutable( items, activeIndex, overIndex)
      });
  }

  const containerStyle = { display: horizontal ? "flex" : '' };

return (
  <DndContext
    sensors={sensors}
    onDragEnd={handleDragEnd}
  >
    <div style={containerStyle}>
        <Droppable id="aaa" 
                   items={items} 
                   itemProp={itemProp} 
                   keyField={keyField} 
                   render={render}/>
    </div>
  </DndContext>
);
}


const ImgBox = ({imgArr})=>{

  const useStyles = makeStyles((theme) => ({
    img:{
      borderRadius: "8px",
      width: '100%'
    }
  }));
  const classes = useStyles();

  return(<>
        <img className={classes.img} src={origUrl+imgArr.url}/>
        </>
  )
}

 function CreateAd({action, newImg}) {
  const dispatch = useDispatch();
  const {acceptedFiles, getRootProps, getInputProps} = useDropzone();
  const [imgArr, setImgArr] = useState([]);
  const [sortImgArr, setSortImgArr] = useState([]);
  const [createAd, setCreateAd] = useState({ title:'', description:'', price:0});
  const [sortOldImgArr, setOldSortImgArr] = useState([]);

  useEffect(() => {
    acceptedFiles.map(file =>action(file));
  }, [acceptedFiles]);

useEffect(() => {
  if(JSON.stringify(newImg) !== JSON.stringify(sortOldImgArr)){
    if(imgArr.length>0){
    setImgArr([...imgArr,...[newImg]])
    setOldSortImgArr(newImg)
    }else{
      setImgArr([newImg])
      setOldSortImgArr(newImg)
    }
  }
}, [newImg]);


useEffect(() => {
  setImgArr([])
},[])




const onclickCreate = ()=>{
  if(sortImgArr.length>0){
    let arrIdImg = sortImgArr.map((el)=> el={_id:el._id})
    dispatch(actioCreateAd({images:arrIdImg,...createAd, tags: [''], address: owner.address}))
  }else{
    dispatch(actioCreateAd({...createAd, tags: [''], address: owner.address}))
  }
}

  
const useStyles = makeStyles((theme) => ({
  root: {
    height: '50vh',
  },
  paper: {
    margin: theme.spacing(8, 4),
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'center',
  },
  input:{
    marginBottom: '3%'
  },
  form: {
    width: '100%', // Fix IE 11 issue.
    marginTop: theme.spacing(1),
  },
  submit: {
    margin: theme.spacing(3, 0, 2),
  },
}));
const owner = useSelector(state => state.promise?.user?.payload)
  const classes = useStyles();

  return (
    <main className="mainAdd">
        <div className="bannerAdd" >
          <div className="conteiner">
            <div className="flex__boxx">
              <div className="imd__flex__boxx">
                <Dnd items={imgArr.filter((el)=>el._id)} render={ImgBox} itemProp="imgArr" keyField="_id"
                    onChange={newArray=>setSortImgArr(newArray)} //ставлю место консоль лога это: setSortImgArr(newArray)
                    />
              </div>
            <section className="container">
              <div {...getRootProps({className: 'dropzone'})}>
                <input {...getInputProps()} />
                <p>Drag 'n' drop some files here, or click to select files</p>
              </div>
              <aside>
                <h4>Files</h4>
              </aside>
            </section>
          
                
              <div className="bannerr">
              <TextField 
              type='number'
              value={createAd.price}
              onChange={(e) => setCreateAd({...createAd, price: +(e.target.value)})}
                className={classes.input} 
                required id="standard-required" 
                label="Price $" 
                />
              <TextField
              value={createAd.title}
              onChange={(e) => setCreateAd({...createAd, title: e.target.value})} 
                className={classes.input} 
                required id="standard-required" 
                label="Title"  
                />
              <TextField 
              value={createAd.description}
              onChange={(e) => setCreateAd({...createAd, description: e.target.value})}
                id="outlined-multiline-static"
                label="Description"
                multiline
                rows={4}
                variant="outlined"
              />
              <Button
              onClick={onclickCreate}
              type="submit"
              fullWidth
              variant="contained"
              color="primary"
              className={classes.submit}
              startIcon={<AddCircleIcon/>}
            >
              create ad
            </Button>
              </div>
                
               
            </div>
          </div>
        </div>
    </main>
  );
}



export default CreateAd