import {useState, useEffect, useRef} from 'react';
import {Provider, useDispatch, useSelector} from 'react-redux';
import { makeStyles } from '@material-ui/core/styles';
import Grid from '@material-ui/core/Grid';
import Card from '@material-ui/core/Card';
import TextField from '@material-ui/core/TextField';
import CardActions from '@material-ui/core/CardActions';
import CardContent from '@material-ui/core/CardContent';
import CardMedia from '@material-ui/core/CardMedia';
import { IconButton, Typography, Button} from '@material-ui/core';
import AddAPhotoIcon from '@material-ui/icons/AddAPhoto';
import DeleteIcon from '@material-ui/icons/Delete';
import CreateIcon from '@material-ui/icons/Create';
import Paper from '@material-ui/core/Paper';
import SearchIcon from '@material-ui/icons/Search';
import InputBase from '@material-ui/core/InputBase';
import VisibilityIcon from '@material-ui/icons/Visibility';
import {origUrl, defaultImg, actioCreateAd, history} from '../../App.js';



const GoodCard = ({good}) => {
    const dispatch = useDispatch();
    const owner = useSelector(state => state.promise?.user?.payload)
    const [indexArr, setindexArr] = useState(0)
    const [createAd, setCreateAd] = useState({ title:good.title?good.title:"", description:good.description?good.description:"", price:good.price?good.price:0});
    const onClickGoods = ()=>{
        dispatch(actioCreateAd({...createAd, _id:good._id, tags: [''], address: owner.address}))
    }
    const useStylesImg = makeStyles({
        button:{
            margin: "3%"
        },
        input:{
            margin: '3px'
        },
        inputDescription:{
            margin: '3px',
            width: '90%'
        },
        mediaButton:{
            display: "flex",
            flexDirection: "column",
            justifyContent: "space-around",
            alignItems: "stretch"
        },
        contentCard:{
            display: "flex",
            flexDirection: "column",
            width: "40%",
        },
        content:{
            width: "60%",
            display: "flex",
            flexWrap: "wrap",
            alignContent: "space-around",
            justifyContent: "space-evenly",
        },
        root: {
            display: "flex",
            margin: "2%",
            marginTop: "15px",
        },
        media: {
            width: "100%",
            height: 280,
        },
    })
    return (
    
      <Card className={useStylesImg().root}>
        <CardContent className={useStylesImg().contentCard}>
            <CardMedia
                className={useStylesImg().media}
                onClick={()=>indexArr=== good.images.length-1?setindexArr(0) :setindexArr(indexArr+1)}
                image={good.images&&good.images[indexArr]?origUrl+good.images[indexArr].url:defaultImg}
                title="Contemplative Reptile"/>
            <Button
                className={useStylesImg().button}
                type="submit"
                fullWidth
                variant="outlined"
                color='primary'
                startIcon={<AddAPhotoIcon />}>
                Сhange photo
            </Button>
        </CardContent>
        <CardContent className={useStylesImg().content}>
        <TextField
              value={createAd.title}
              onChange={(e) => setCreateAd({...createAd, title: e.target.value})} 
                className={useStylesImg().input} 
                required id="standard-required" 
                label="Title"  
                />
            <TextField 
              type='number'
              value={createAd.price}
              onChange={(e) => setCreateAd({...createAd, price: +(e.target.value)})}
                className={useStylesImg().input} 
                required id="standard-required" 
                label="Price $" 
                />
                <TextField 
              value={createAd.description}
              onChange={(e) => setCreateAd({...createAd, description: e.target.value})}
                id="outlined-multiline-static"
                label="Description"
                className={useStylesImg().inputDescription} 
                multiline
                rows={4}
                variant="outlined"
              />
          </CardContent>
          <CardActions className={useStylesImg().mediaButton}>
          <Button onClick={onClickGoods} color="inherit" variant="outlined" ><CreateIcon/> Сhange </Button>
          <Button onClick={()=> history.push("/good/"+good._id)} color="inherit" variant="outlined" ><VisibilityIcon/> Open </Button>
          <Button color="secondary" variant="contained"><DeleteIcon/> Delete </Button>
        </CardActions>
      
      </Card>
    
  )}
  
  const CardsUser = ({loadData, productsUser=[], stat}) => {
    const id = useSelector(state => state.auth?.payload?.sub?.id)
    useEffect(() => {
      loadData(id)
    },[id])
    const useStyles = makeStyles((theme) => ({
      root: {
        padding: '2px 4px',
        display: 'flex',
        alignItems: 'center',
        marginTop: '55px',
        marginBottom: '20px',
        width: '70%',
        marginLeft: '16%'
      },
      rootGrid:{
        display: 'flex',
        flexDirection: 'column',
      },
      input: {
        marginLeft: theme.spacing(1),
        flex: 1,
      },
      iconButton: {
        padding: 10,
      },
    }));
    const classes = useStyles()
    if(stat==='PENDING'){
      return(
        <img className='preloader' src='https://i.pinimg.com/originals/c8/a1/76/c8a1765ffc8243f3a7b176c0ca84c3c1.gif' />
      )
    }
    return(<>
      <Paper component="form" className={classes.root}>
      <IconButton className={classes.iconButton} aria-label="search">
          <SearchIcon />
        </IconButton>
        <InputBase
          className={classes.input}
          placeholder="Search"/>
      </Paper>
    <Grid className={classes.rootGrid} container spacing={4} >
    {productsUser.map(el=><GoodCard key={el._id} good={el} />)}
    </Grid></>)
  }


  export default CardsUser